import contextlib
from collections.abc import Generator

from clippy.remote import Remote, RemoteHosts


class KubernetesWorker:
    def __init__(self, name: str, cluster: str, remote_host: RemoteHosts, kubectl_host: RemoteHosts) -> None:
        self._name = name
        self._cluster = cluster
        self._remote_host = remote_host
        self._kubectl_host = kubectl_host

    def depool(self, grace_period: int = 60) -> None:
        self._kubectl_host.execute(
            command=f"KUBECONFIG=/etc/kubernetes/config.yaml kubectl --context={self._cluster} drain {self._name} --ignore-daemonsets --delete-emptydir-data --grace-period={grace_period}",
            show_output=True,
        )

    def repool(self):
        self._kubectl_host.execute(
            command=f"KUBECONFIG=/etc/kubernetes/config.yaml kubectl --context={self._cluster} uncordon {self._name}",
            show_output=True,
        )

    @contextlib.contextmanager
    def depooled(self, grace_period: int = 60) -> Generator[None, None, None]:
        self.depool(grace_period=grace_period)
        yield
        self.repool()


class KubernetesCluster:
    def __init__(self, name: str, remote: Remote, kubectl_host: RemoteHosts) -> None:
        self._name = name
        self._remote = remote
        self._kubectl_host = kubectl_host

    def worker(self, fqdn: str) -> KubernetesWorker:
        name = fqdn.split(".")[0]
        return KubernetesWorker(
            name=name,
            cluster=self._name,
            remote_host=self._remote.query(f"D{{{fqdn}}}"),
            kubectl_host=self._kubectl_host,
        )
